(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.vueCoeImage = {})));
}(this, (function (exports) { 'use strict';

  var image = {
    name: 'vue-coe-image',
    props: {
      src: {
        type: String,
        required: true
      },
      fallback: String,
      delay: {
        type: [String, Number],
        default: 0
      },
      blurLevel: {
        type: Number,
        default: 30
      },
      duration: {
        type: Number,
        default: 1000
      },
      intersectionOptions: {
        type: Object,
        default: function _default() {
          return {};
        }
      }
    },
    data: function data() {
      return {
        rate: 1,
        observer: null,
        intersected: false,
        hasError: false
      };
    },
    watch: {
      src: function src(x, y) {
        if (x !== y) this.hasError = false;
      }
    },
    mounted: function mounted() {
      var _this = this;

      setTimeout(function () {
        _this.observer = new IntersectionObserver(function (entries) {
          var image = entries[0];

          if (image.isIntersecting) {
            _this.intersected = true;

            _this.observer.disconnect();

            _this.$emit('intersect');
          }
        }, _this.intersectionOptions);

        _this.observer.observe(_this.$el);
      }, this.delay);
    },
    computed: {
      image: function image() {
        var fallbackSrc = this.fallback;
        var imageSource = this.hasError ? fallbackSrc : this.src;
        return this.intersected && imageSource;
      },
      deviation: function deviation() {
        return this.blurLevel * this.rate;
      }
    },
    methods: {
      animate: function animate() {
        var _this2 = this;

        var start = Date.now() + this.duration;

        var step = function step() {
          var remaining = start - Date.now();

          if (remaining < 0) {
            _this2.rate = 0;
          } else {
            _this2.rate = remaining / _this2.duration;
            requestAnimationFrame(step);
          }
        };

        requestAnimationFrame(step);
      },
      handleError: function handleError() {
        if ("development" !== 'production' && !this.fallback) {
          console.warn('an error occured during the image loading');
        }

        this.hasError = true;
        this.$emit('error');
      }
    },
    render: function render(h) {
      var feGaussianBlur = h('feGaussianBlur', {
        attrs: {
          in: 'SourceGraphic',
          stdDeviation: this.deviation
        }
      });
      var filter = h('filter', {
        attrs: {
          id: 'blur'
        }
      }, [feGaussianBlur]);
      var defs = h('defs', null, [filter]);
      var svg = h('svg', {
        class: ['filter-defs-svg hidden'],
        attrs: {
          xmlns: 'http://www.w3.org/2000/svg',
          version: '1.1'
        }
      }, [defs]);
      var image = h('img', {
        attrs: {
          src: this.image
        },
        class: 'lazy-load-image',
        on: {
          load: this.animate,
          error: this.handleError
        }
      });
      return h('div', {
        class: 'vue-coe-image'
      }, [svg, image]);
    },
    destroyed: function destroyed() {
      this.observer.disconnect();
    }
  };
  var VueCoeImage = {
    install: function install(Vue) {
      var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref$name = _ref.name,
          name = _ref$name === void 0 ? 'vue-coe-image' : _ref$name;

      Vue.component(name, this);
    }
  };

  exports.default = image;
  exports.VueCoeImage = VueCoeImage;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
